@@ -79,10 +79,15 @@ Q_IMPORT_PLUGIN(qico)
 
 // Signal handlers
 #if defined(Q_OS_UNIX) || defined(STACKTRACE_WIN)
-void sigintHandler(int);
-void sigtermHandler(int);
-void sigsegvHandler(int);
-void sigabrtHandler(int);
+void sigNormalHandler(int signum);
+void sigAbnormalHandler(int signum);
+// sys_signame[] is only defined in BSD
+const char *sysSigName[] = {
+    "", "SIGHUP", "SIGINT", "SIGQUIT", "SIGILL", "SIGTRAP", "SIGABRT", "SIGBUS", "SIGFPE", "SIGKILL",
+    "SIGUSR1", "SIGSEGV", "SIGUSR2", "SIGPIPE", "SIGALRM", "SIGTERM", "SIGSTKFLT", "SIGCHLD", "SIGCONT", "SIGSTOP",
+    "SIGTSTP", "SIGTTIN", "SIGTTOU", "SIGURG", "SIGXCPU", "SIGXFSZ", "SIGVTALRM", "SIGPROF", "SIGWINCH", "SIGIO",
+    "SIGPWR", "SIGUNUSED"
+};
 #endif
 
 struct QBtCommandLineParameters
@@ -210,6 +215,21 @@ int main(int argc, char *argv[])
         return EXIT_SUCCESS;
     }
 
+#if defined(Q_OS_WIN) && defined(QBT_USES_QT5)
+    // This affects only Windows apparently and Qt5.
+    // When QNetworkAccessManager is instantiated it regularly starts polling
+    // the network interfaces to see what's available and their status.
+    // This polling creates jitter and high ping with wifi interfaces.
+    // So here we disable it for lack of better measure.
+    // It will also spew this message in the console: QObject::startTimer: Timers cannot have negative intervals
+    // For more info see:
+    // 1. https://github.com/qbittorrent/qBittorrent/issues/4209
+    // 2. https://bugreports.qt.io/browse/QTBUG-40332
+    // 3. https://bugreports.qt.io/browse/QTBUG-46015
+
+    qputenv("QT_BEARER_POLL_TIMEOUT", QByteArray::number(-1));
+#endif
+
 #ifndef DISABLE_GUI
     if (!upgrade()) return EXIT_FAILURE;
 #else
@@ -240,10 +260,10 @@ int main(int argc, char *argv[])
 #endif
 
 #if defined(Q_OS_UNIX) || defined(STACKTRACE_WIN)
-    signal(SIGABRT, sigabrtHandler);
-    signal(SIGTERM, sigtermHandler);
-    signal(SIGINT, sigintHandler);
-    signal(SIGSEGV, sigsegvHandler);
+    signal(SIGINT, sigNormalHandler);
+    signal(SIGTERM, sigNormalHandler);
+    signal(SIGABRT, sigAbnormalHandler);
+    signal(SIGSEGV, sigAbnormalHandler);
 #endif
 
     return app->exec(params.torrents);
@@ -303,58 +323,41 @@ QBtCommandLineParameters parseCommandLine()
 }
 
 #if defined(Q_OS_UNIX) || defined(STACKTRACE_WIN)
-void sigintHandler(int)
+void sigNormalHandler(int signum)
 {
-    signal(SIGINT, 0);
-    qDebug("Catching SIGINT, exiting cleanly");
-    qApp->exit();
-}
-
-void sigtermHandler(int)
-{
-    signal(SIGTERM, 0);
-    qDebug("Catching SIGTERM, exiting cleanly");
-    qApp->exit();
-}
-
-void sigsegvHandler(int)
-{
-    signal(SIGABRT, 0);
-    signal(SIGSEGV, 0);
 #if !defined Q_OS_WIN && !defined Q_OS_HAIKU
-    std::cerr << "\n\n*************************************************************\n";
-    std::cerr << "Catching SIGSEGV, please report a bug at http://bug.qbittorrent.org\nand provide the following backtrace:\n";
-    std::cerr << "qBittorrent version: " << VERSION << std::endl;
-    print_stacktrace();
-#else
+    const char str1[] = "Catching signal: ";
+    const char *sigName = sysSigName[signum];
+    const char str2[] = "\nExiting cleanly\n";
+    write(STDERR_FILENO, str1, strlen(str1));
+    write(STDERR_FILENO, sigName, strlen(sigName));
+    write(STDERR_FILENO, str2, strlen(str2));
+#endif // !defined Q_OS_WIN && !defined Q_OS_HAIKU
+    signal(signum, SIG_DFL);
+    qApp->exit();  // unsafe, but exit anyway
+}
+
+void sigAbnormalHandler(int signum)
+{
+#if !defined Q_OS_WIN && !defined Q_OS_HAIKU
+    const char str1[] = "\n\n*************************************************************\nCatching signal: ";
+    const char *sigName = sysSigName[signum];
+    const char str2[] = "\nPlease file a bug report at http://bug.qbittorrent.org and provide the following information:\n\n"
+    "qBittorrent version: " VERSION "\n";
+    write(STDERR_FILENO, str1, strlen(str1));
+    write(STDERR_FILENO, sigName, strlen(sigName));
+    write(STDERR_FILENO, str2, strlen(str2));
+    print_stacktrace();  // unsafe
+#endif // !defined Q_OS_WIN && !defined Q_OS_HAIKU
 #ifdef STACKTRACE_WIN
-    StraceDlg dlg;
+    StraceDlg dlg;  // unsafe
     dlg.setStacktraceString(straceWin::getBacktrace());
     dlg.exec();
-#endif
-#endif
-    raise(SIGSEGV);
+#endif // STACKTRACE_WIN
+    signal(signum, SIG_DFL);
+    raise(signum);
 }
-
-void sigabrtHandler(int)
-{
-    signal(SIGABRT, 0);
-    signal(SIGSEGV, 0);
-#if !defined Q_OS_WIN && !defined Q_OS_HAIKU
-    std::cerr << "\n\n*************************************************************\n";
-    std::cerr << "Catching SIGABRT, please report a bug at http://bug.qbittorrent.org\nand provide the following backtrace:\n";
-    std::cerr << "qBittorrent version: " << VERSION << std::endl;
-    print_stacktrace();
-#else
-#ifdef STACKTRACE_WIN
-    StraceDlg dlg;
-    dlg.setStacktraceString(straceWin::getBacktrace());
-    dlg.exec();
-#endif
-#endif
-    raise(SIGABRT);
-}
-#endif
+#endif // defined(Q_OS_UNIX) || defined(STACKTRACE_WIN)
 
 #ifndef DISABLE_GUI
 void showSplashScreen()
@@ -365,9 +368,9 @@ void showSplashScreen()
     painter.setPen(QPen(Qt::white));
     painter.setFont(QFont("Arial", 22, QFont::Black));
     painter.drawText(224 - painter.fontMetrics().width(version), 270, version);
-    QSplashScreen *splash = new QSplashScreen(splash_img, Qt::WindowStaysOnTopHint);
-    QTimer::singleShot(1500, splash, SLOT(deleteLater()));
+    QSplashScreen *splash = new QSplashScreen(splash_img);
     splash->show();
+    QTimer::singleShot(1500, splash, SLOT(deleteLater()));
     qApp->processEvents();
 }
 #endif
