@@ -32,12 +32,15 @@
 
 #include <QFile>
 #include <QHash>
+#include <QMap>
 #include <QPointer>
 #include <QVector>
 #include <QMutex>
 #include <QWaitCondition>
 #include <QNetworkConfigurationManager>
 
+#include <libtorrent/version.hpp>
+
 #include "base/tristatebool.h"
 #include "base/types.h"
 #include "torrentinfo.h"
@@ -45,13 +48,24 @@
 namespace libtorrent
 {
     class session;
+    struct torrent_handle;
     class entry;
     struct add_torrent_params;
     struct pe_settings;
-    struct proxy_settings;
     struct session_settings;
     struct session_status;
 
+#if LIBTORRENT_VERSION_NUM < 10100
+    struct proxy_settings;
+#else
+    namespace aux
+    {
+        struct proxy_settings;
+    }
+
+    typedef aux::proxy_settings proxy_settings;
+#endif
+
     class alert;
     struct torrent_alert;
     struct state_update_alert;
@@ -86,6 +100,7 @@ namespace libtorrent
     struct external_ip_alert;
 }
 
+class QThread;
 class QTimer;
 class QStringList;
 class QString;
@@ -95,8 +110,20 @@ template<typename T> class QList;
 class FilterParserThread;
 class BandwidthScheduler;
 class Statistics;
+class ResumeDataSavingManager;
+class SettingsStorage;
 
-typedef QPair<QString, QString> QStringPair;
+enum MaxRatioAction
+{
+    Pause,
+    Remove
+};
+
+enum TorrentExportFolder
+{
+    Regular,
+    Finished
+};
 
 namespace BitTorrent
 {
@@ -112,17 +139,15 @@ namespace BitTorrent
     struct AddTorrentParams
     {
         QString name;
-        QString label;
+        QString category;
         QString savePath;
-        bool disableTempPath; // e.g. for imported torrents
-        bool sequential;
+        bool disableTempPath = false; // e.g. for imported torrents
+        bool sequential = false;
         TriStateBool addForced;
         TriStateBool addPaused;
         QVector<int> filePriorities; // used if TorrentInfo is set
-        bool ignoreShareRatio;
-        bool skipChecking;
-
-        AddTorrentParams();
+        bool ignoreShareRatio = false;
+        bool skipChecking = false;
     };
 
     struct TorrentStatusReport
@@ -152,11 +177,50 @@ namespace BitTorrent
         bool isPexEnabled() const;
         bool isQueueingEnabled() const;
         qreal globalMaxRatio() const;
-        bool isTempPathEnabled() const;
         bool isAppendExtensionEnabled() const;
-        bool useAppendLabelToSavePath() const;
+
         QString defaultSavePath() const;
+        void setDefaultSavePath(QString path);
         QString tempPath() const;
+        void setTempPath(QString path);
+        bool isTempPathEnabled() const;
+        void setTempPathEnabled(bool enabled);
+        QString torrentTempPath(const InfoHash &hash) const;
+
+        static bool isValidCategoryName(const QString &name);
+        // returns category itself and all top level categories
+        static QStringList expandCategory(const QString &category);
+
+        QStringList categories() const;
+        QString categorySavePath(const QString &categoryName) const;
+        bool addCategory(const QString &name, const QString &savePath = "");
+        bool editCategory(const QString &name, const QString &savePath);
+        bool removeCategory(const QString &name);
+        bool isSubcategoriesEnabled() const;
+        void setSubcategoriesEnabled(bool value);
+
+        // Torrent Management Mode subsystem (TMM)
+        //
+        // Each torrent can be either in Manual mode or in Automatic mode
+        // In Manual Mode various torrent properties are set explicitly(eg save path)
+        // In Automatic Mode various torrent properties are set implicitly(eg save path)
+        //     based on the associated category.
+        // In Automatic Mode torrent save path can be changed in following cases:
+        //     1. Default save path changed
+        //     2. Torrent category save path changed
+        //     3. Torrent category changed
+        //     (unless otherwise is specified)
+        bool isAutoTMMDisabledByDefault() const;
+        void setAutoTMMDisabledByDefault(bool value);
+        bool isDisableAutoTMMWhenCategoryChanged() const;
+        void setDisableAutoTMMWhenCategoryChanged(bool value);
+        bool isDisableAutoTMMWhenDefaultSavePathChanged() const;
+        void setDisableAutoTMMWhenDefaultSavePathChanged(bool value);
+        bool isDisableAutoTMMWhenCategorySavePathChanged() const;
+        void setDisableAutoTMMWhenCategorySavePathChanged(bool value);
+
+        bool isAddTorrentPaused() const;
+        void setAddTorrentPaused(bool value);
 
         TorrentHandle *findTorrent(const InfoHash &hash) const;
         QHash<InfoHash, TorrentHandle *> torrents() const;
@@ -171,6 +235,9 @@ namespace BitTorrent
         int uploadRateLimit() const;
         bool isListening() const;
 
+        MaxRatioAction maxRatioAction() const;
+        void setMaxRatioAction(MaxRatioAction act);
+
         void changeSpeedLimitMode(bool alternative);
         void setDownloadRateLimit(int rate);
         void setUploadRateLimit(int rate);
@@ -183,7 +250,7 @@ namespace BitTorrent
         bool addTorrent(QString source, const AddTorrentParams &params = AddTorrentParams());
         bool addTorrent(const TorrentInfo &torrentInfo, const AddTorrentParams &params = AddTorrentParams());
         bool deleteTorrent(const QString &hash, bool deleteLocalFiles = false);
-        bool loadMetadata(const QString &magnetUri);
+        bool loadMetadata(const MagnetUri &magnetUri);
         bool cancelLoadMetadata(const InfoHash &hash);
 
         void recursiveTorrentDownload(const InfoHash &hash);
@@ -195,7 +262,8 @@ namespace BitTorrent
         // TorrentHandle interface
         void handleTorrentRatioLimitChanged(TorrentHandle *const torrent);
         void handleTorrentSavePathChanged(TorrentHandle *const torrent);
-        void handleTorrentLabelChanged(TorrentHandle *const torrent, const QString &oldLabel);
+        void handleTorrentCategoryChanged(TorrentHandle *const torrent, const QString &oldCategory);
+        void handleTorrentSavingModeChanged(TorrentHandle *const torrent);
         void handleTorrentMetadataReceived(TorrentHandle *const torrent);
         void handleTorrentPaused(TorrentHandle *const torrent);
         void handleTorrentResumed(TorrentHandle *const torrent);
@@ -217,13 +285,15 @@ namespace BitTorrent
         void torrentsUpdated();
         void addTorrentFailed(const QString &error);
         void torrentAdded(BitTorrent::TorrentHandle *const torrent);
+        void torrentNew(BitTorrent::TorrentHandle *const torrent);
         void torrentAboutToBeRemoved(BitTorrent::TorrentHandle *const torrent);
         void torrentPaused(BitTorrent::TorrentHandle *const torrent);
         void torrentResumed(BitTorrent::TorrentHandle *const torrent);
         void torrentFinished(BitTorrent::TorrentHandle *const torrent);
         void torrentFinishedChecking(BitTorrent::TorrentHandle *const torrent);
         void torrentSavePathChanged(BitTorrent::TorrentHandle *const torrent);
-        void torrentLabelChanged(BitTorrent::TorrentHandle *const torrent, const QString &oldLabel);
+        void torrentCategoryChanged(BitTorrent::TorrentHandle *const torrent, const QString &oldCategory);
+        void torrentSavingModeChanged(BitTorrent::TorrentHandle *const torrent);
         void allTorrentsFinished();
         void metadataLoaded(const BitTorrent::TorrentInfo &info);
         void torrentMetadataLoaded(BitTorrent::TorrentHandle *const torrent);
@@ -241,6 +311,9 @@ namespace BitTorrent
         void trackerlessStateChanged(BitTorrent::TorrentHandle *const torrent, bool trackerless);
         void downloadFromUrlFailed(const QString &url, const QString &reason);
         void downloadFromUrlFinished(const QString &url);
+        void categoryAdded(const QString &categoryName);
+        void categoryRemoved(const QString &categoryName);
+        void subcategoriesSupportChanged();
 
     private slots:
         void configure();
@@ -274,8 +347,6 @@ namespace BitTorrent
         void adjustLimits(libtorrent::session_settings &sessionSettings);
         const QStringList getListeningIPs();
         void setListeningPort();
-        void setDefaultSavePath(const QString &path);
-        void setDefaultTempPath(const QString &path = QString());
         void preAllocateAllFiles(bool b);
         void setMaxConnectionsPerTorrent(int max);
         void setMaxUploadsPerTorrent(int max);
@@ -283,13 +354,13 @@ namespace BitTorrent
         void enableDHT(bool enable);
         void changeSpeedLimitMode_impl(bool alternative);
 
-        void setAppendLabelToSavePath(bool append);
         void setAppendExtension(bool append);
 
         void startUpTorrents();
         bool addTorrent_impl(AddTorrentData addData, const MagnetUri &magnetUri,
-                             const TorrentInfo &torrentInfo = TorrentInfo(),
+                             TorrentInfo torrentInfo = TorrentInfo(),
                              const QByteArray &fastresumeData = QByteArray());
+        bool findIncompleteFiles(TorrentInfo &torrentInfo, QString &savePath) const;
 
         void updateRatioTimer();
         void exportTorrentFile(TorrentHandle *const torrent, TorrentExportFolder folder = TorrentExportFolder::Regular);
@@ -313,13 +384,16 @@ namespace BitTorrent
         void handleListenFailedAlert(libtorrent::listen_failed_alert *p);
         void handleExternalIPAlert(libtorrent::external_ip_alert *p);
 
+        void createTorrentHandle(const libtorrent::torrent_handle &nativeHandle);
+
         void saveResumeData();
-        bool writeResumeDataFile(TorrentHandle *const torrent, const libtorrent::entry &data);
 
+#if LIBTORRENT_VERSION_NUM < 10100
         void dispatchAlerts(std::auto_ptr<libtorrent::alert> alertPtr);
-        void getPendingAlerts(QVector<libtorrent::alert *> &out, ulong time = 0);
+#endif
+        void getPendingAlerts(std::vector<libtorrent::alert *> &out, ulong time = 0);
 
-        AddTorrentData addDataFromParams(const AddTorrentParams &params);
+        SettingsStorage *m_settings;
 
         // BitTorrent
         libtorrent::session *m_nativeSession;
@@ -334,10 +408,9 @@ namespace BitTorrent
         qreal m_globalMaxRatio;
         int m_numResumeData;
         int m_extraLimit;
-        bool m_appendLabelToSavePath;
         bool m_appendExtension;
         uint m_refreshInterval;
-        MaxRatioAction m_highRatioAction;
+        MaxRatioAction m_maxRatioAction;
         QList<BitTorrent::TrackerEntry> m_additionalTrackers;
         QString m_defaultSavePath;
         QString m_tempPath;
@@ -355,16 +428,22 @@ namespace BitTorrent
         QPointer<BandwidthScheduler> m_bwScheduler;
         // Tracker
         QPointer<Tracker> m_tracker;
+        // fastresume data writing thread
+        QThread *m_ioThread;
+        ResumeDataSavingManager *m_resumeDataSavingManager;
 
         QHash<InfoHash, TorrentInfo> m_loadedMetadata;
         QHash<InfoHash, TorrentHandle *> m_torrents;
         QHash<InfoHash, AddTorrentData> m_addingTorrents;
         QHash<QString, AddTorrentParams> m_downloadedTorrents;
         TorrentStatusReport m_torrentStatusReport;
+        QStringMap m_categories;
 
+#if LIBTORRENT_VERSION_NUM < 10100
         QMutex m_alertsMutex;
         QWaitCondition m_alertsWaitCondition;
-        QVector<libtorrent::alert *> m_alerts;
+        std::vector<libtorrent::alert *> m_alerts;
+#endif
 
         QNetworkConfigurationManager m_networkManager;
 
