@@ -40,10 +40,14 @@
 #include <libtorrent/alert_types.hpp>
 #include <libtorrent/create_torrent.hpp>
 #include <libtorrent/magnet_uri.hpp>
+#if LIBTORRENT_VERSION_NUM >= 10100
+#include <libtorrent/time.hpp>
+#endif
+
 #include <boost/bind.hpp>
 
 #ifdef Q_OS_WIN
-#include <Windows.h>
+#include <windows.h>
 #endif
 
 #include "base/logger.h"
@@ -56,15 +60,36 @@
 #include "trackerentry.h"
 #include "torrenthandle.h"
 
-static const char QB_EXT[] = ".!qB";
+const QString QB_EXT {".!qB"};
 
 namespace libt = libtorrent;
 using namespace BitTorrent;
 
-// TrackerInfo
+// AddTorrentData
 
-TrackerInfo::TrackerInfo()
-    : numPeers(0)
+AddTorrentData::AddTorrentData()
+    : resumed(false)
+    , disableTempPath(false)
+    , sequential(false)
+    , hasSeedStatus(false)
+    , skipChecking(false)
+    , ratioLimit(TorrentHandle::USE_GLOBAL_RATIO)
+{
+}
+
+AddTorrentData::AddTorrentData(const AddTorrentParams &params)
+    : resumed(false)
+    , name(params.name)
+    , category(params.category)
+    , savePath(params.savePath)
+    , disableTempPath(params.disableTempPath)
+    , sequential(params.sequential)
+    , hasSeedStatus(params.skipChecking) // do not react on 'torrent_finished_alert' when skipping
+    , skipChecking(params.skipChecking)
+    , addForced(params.addForced)
+    , addPaused(params.addPaused)
+    , filePriorities(params.filePriorities)
+    , ratioLimit(params.ignoreShareRatio ? TorrentHandle::NO_RATIO_LIMIT : TorrentHandle::USE_GLOBAL_RATIO)
 {
 }
 
@@ -174,9 +199,10 @@ TorrentHandle::TorrentHandle(Session *session, const libtorrent::torrent_handle
     , m_nativeHandle(nativeHandle)
     , m_state(TorrentState::Unknown)
     , m_renameCount(0)
+    , m_useAutoTMM(data.savePath.isEmpty())
     , m_name(data.name)
     , m_savePath(Utils::Fs::toNativePath(data.savePath))
-    , m_label(data.label)
+    , m_category(data.category)
     , m_hasSeedStatus(data.hasSeedStatus)
     , m_ratioLimit(data.ratioLimit)
     , m_tempPathDisabled(data.disableTempPath)
@@ -184,15 +210,14 @@ TorrentHandle::TorrentHandle(Session *session, const libtorrent::torrent_handle
     , m_pauseAfterRecheck(false)
     , m_needSaveResumeData(false)
 {
-    initialize();
+    if (m_useAutoTMM)
+        m_savePath = Utils::Fs::toNativePath(m_session->categorySavePath(m_category));
 
-    if (!data.resumed) {
+    updateStatus();
+    m_hash = InfoHash(m_nativeStatus.info_hash);
+
+    if (!data.resumed)
         setSequentialDownload(data.sequential);
-        if (hasMetadata()) {
-            if (m_session->isAppendExtensionEnabled())
-                appendExtensionsToIncompleteFiles();
-        }
-    }
 }
 
 TorrentHandle::~TorrentHandle() {}
@@ -301,6 +326,22 @@ QString TorrentHandle::contentPath(bool actual) const
         return rootPath(actual);
 }
 
+bool TorrentHandle::isAutoTMMEnabled() const
+{
+    return m_useAutoTMM;
+}
+
+void TorrentHandle::setAutoTMMEnabled(bool enabled)
+{
+    if (m_useAutoTMM == enabled) return;
+
+    m_useAutoTMM = enabled;
+    m_session->handleTorrentSavingModeChanged(this);
+
+    if (m_useAutoTMM)
+        move_impl(m_session->categorySavePath(m_category));
+}
+
 QString TorrentHandle::nativeActualSavePath() const
 {
     return Utils::String::fromStdString(m_nativeStatus.save_path);
@@ -433,7 +474,7 @@ bool TorrentHandle::connectPeer(const PeerAddress &peerAddress)
     libt::address addr = libt::address::from_string(Utils::String::toStdString(peerAddress.ip.toString()), ec);
     if (ec) return false;
 
-    libt::asio::ip::tcp::endpoint ep(addr, peerAddress.port);
+    boost::asio::ip::tcp::endpoint ep(addr, peerAddress.port);
     SAFE_CALL_BOOL(connect_peer, ep);
 }
 
@@ -478,9 +519,22 @@ qreal TorrentHandle::progress() const
     return progress;
 }
 
-QString TorrentHandle::label() const
+QString TorrentHandle::category() const
 {
-    return m_label;
+    return m_category;
+}
+
+bool TorrentHandle::belongsToCategory(const QString &category) const
+{
+    if (m_category.isEmpty()) return category.isEmpty();
+    if (!Session::isValidCategoryName(category)) return false;
+
+    if (m_category == category) return true;
+
+    if (m_session->isSubcategoriesEnabled() && m_category.startsWith(category + "/"))
+        return true;
+
+    return false;
 }
 
 QDateTime TorrentHandle::addedTime() const
@@ -543,25 +597,6 @@ QStringList TorrentHandle::absoluteFilePathsUnwanted() const
     return res;
 }
 
-QPair<int, int> TorrentHandle::fileExtremityPieces(int index) const
-{
-    if (!hasMetadata()) return qMakePair(-1, -1);
-
-    const int numPieces = piecesCount();
-    const qlonglong pieceSize = pieceLength();
-
-    // Determine the first and last piece of the file
-    int firstPiece = floor((m_torrentInfo.fileOffset(index) + 1) / (float) pieceSize);
-    Q_ASSERT((firstPiece >= 0) && (firstPiece < numPieces));
-
-    int numPiecesInFile = ceil(fileSize(index) / (float) pieceSize);
-    int lastPiece = firstPiece + numPiecesInFile - 1;
-    Q_ASSERT((lastPiece >= 0) && (lastPiece < numPieces));
-
-    Q_UNUSED(numPieces)
-    return qMakePair(firstPiece, lastPiece);
-}
-
 QVector<int> TorrentHandle::filePriorities() const
 {
     std::vector<int> fp;
@@ -679,13 +714,13 @@ bool TorrentHandle::hasFirstLastPiecePriority() const
     std::vector<int> fp;
     SAFE_GET(fp, file_priorities);
 
-    QPair<int, int> extremities;
+    TorrentInfo::PieceRange extremities;
     bool found = false;
     int count = static_cast<int>(fp.size());
     for (int i = 0; i < count; ++i) {
         const QString ext = Utils::Fs::fileExtension(filePath(i));
         if (Utils::Misc::isPreviewable(ext) && (fp[i] > 0)) {
-            extremities = fileExtremityPieces(i);
+            extremities = info().filePieces(i);
             found = true;
             break;
         }
@@ -695,8 +730,8 @@ bool TorrentHandle::hasFirstLastPiecePriority() const
 
     int first = 0;
     int last = 0;
-    SAFE_GET(first, piece_priority, extremities.first);
-    SAFE_GET(last, piece_priority, extremities.second);
+    SAFE_GET(first, piece_priority, extremities.first());
+    SAFE_GET(last, piece_priority, extremities.last());
 
     return ((first == 7) && (last == 7));
 }
@@ -847,7 +882,7 @@ qulonglong TorrentHandle::eta() const
 
 QVector<qreal> TorrentHandle::filesProgress() const
 {
-    std::vector<libt::size_type> fp;
+    std::vector<boost::int64_t> fp;
     QVector<qreal> result;
     SAFE_CALL(file_progress, fp, libt::torrent_handle::piece_granularity);
 
@@ -1022,9 +1057,9 @@ qreal TorrentHandle::maxRatio(bool *usesGlobalRatio) const
 
 qreal TorrentHandle::realRatio() const
 {
-    libt::size_type upload = m_nativeStatus.all_time_upload;
+    boost::int64_t upload = m_nativeStatus.all_time_upload;
     // special case for a seeder who lost its stats, also assume nobody will import a 99% done torrent
-    libt::size_type download = (m_nativeStatus.all_time_download < m_nativeStatus.total_done * 0.01) ? m_nativeStatus.total_done : m_nativeStatus.all_time_download;
+    boost::int64_t download = (m_nativeStatus.all_time_download < m_nativeStatus.total_done * 0.01) ? m_nativeStatus.total_done : m_nativeStatus.all_time_download;
 
     if (download == 0)
         return (upload == 0) ? 0.0 : MAX_RATIO;
@@ -1066,7 +1101,11 @@ int TorrentHandle::connectionsLimit() const
 
 qlonglong TorrentHandle::nextAnnounce() const
 {
+#if LIBTORRENT_VERSION_NUM < 10100
     return m_nativeStatus.next_announce.total_seconds();
+#else
+    return libt::duration_cast<libt::seconds>(m_nativeStatus.next_announce).count();
+#endif
 }
 
 void TorrentHandle::setName(const QString &name)
@@ -1077,17 +1116,47 @@ void TorrentHandle::setName(const QString &name)
     }
 }
 
-void TorrentHandle::setLabel(const QString &label)
+bool TorrentHandle::setCategory(const QString &category)
 {
-    if (m_label != label) {
-        QString oldLabel = m_label;
-        m_label = label;
+    if (m_category != category) {
+        if (!category.isEmpty()) {
+            if (!Session::isValidCategoryName(category)) return false;
+            if (!m_session->categories().contains(category))
+                if (!m_session->addCategory(category))
+                    return false;
+        }
+
+        QString oldCategory = m_category;
+        m_category = category;
         m_needSaveResumeData = true;
-        m_session->handleTorrentLabelChanged(this, oldLabel);
+        m_session->handleTorrentCategoryChanged(this, oldCategory);
+
+        if (m_useAutoTMM) {
+            if (!m_session->isDisableAutoTMMWhenCategoryChanged())
+                move_impl(m_session->categorySavePath(m_category));
+            else
+                setAutoTMMEnabled(false);
+        }
     }
+
+    return true;
 }
 
 void TorrentHandle::move(QString path)
+{
+    m_useAutoTMM = false;
+    m_session->handleTorrentSavingModeChanged(this);
+
+    path = Utils::Fs::fromNativePath(path.trimmed());
+    if (path.isEmpty())
+        path = m_session->defaultSavePath();
+    if (!path.endsWith('/'))
+        path += '/';
+
+    move_impl(path);
+}
+
+void TorrentHandle::move_impl(QString path)
 {
     path = Utils::Fs::toNativePath(path);
     if (path == savePath()) return;
@@ -1143,6 +1212,8 @@ void TorrentHandle::setFirstLastPiecePriority(bool b)
 
     std::vector<int> fp;
     SAFE_GET(fp, file_priorities);
+    std::vector<int> pp;
+    SAFE_GET(pp, piece_priorities);
 
     // Download first and last pieces first for all media files in the torrent
     int nbfiles = static_cast<int>(fp.size());
@@ -1151,14 +1222,22 @@ void TorrentHandle::setFirstLastPiecePriority(bool b)
         const QString ext = Utils::Fs::fileExtension(path);
         if (Utils::Misc::isPreviewable(ext) && (fp[index] > 0)) {
             qDebug() << "File" << path << "is previewable, toggle downloading of first/last pieces first";
+
             // Determine the priority to set
             int prio = b ? 7 : fp[index];
 
-            QPair<int, int> extremities = fileExtremityPieces(index);
-            SAFE_CALL(piece_priority, extremities.first, prio);
-            SAFE_CALL(piece_priority, extremities.second, prio);
+            TorrentInfo::PieceRange extremities = info().filePieces(index);
+
+            // worst case: AVI index = 1% of total file size (at the end of the file)
+            int nNumPieces = ceil(fileSize(index) * 0.01 / pieceLength());
+            for (int i = 0; i < nNumPieces; ++i) {
+                pp[extremities.first() + i] = prio;
+                pp[extremities.last() - i] = prio;
+            }
         }
     }
+
+    SAFE_CALL(prioritize_pieces, pp);
 }
 
 void TorrentHandle::toggleFirstLastPiecePriority()
@@ -1205,10 +1284,6 @@ void TorrentHandle::moveStorage(const QString &newPath)
         if (QDir(oldPath) == QDir(newPath)) return;
 
         qDebug("move storage: %s to %s", qPrintable(oldPath), qPrintable(newPath));
-        // Create destination directory if necessary
-        // or move_storage() will fail...
-        QDir().mkpath(newPath);
-
         try {
             // Actually move the storage
             m_nativeHandle.move_storage(newPath.toUtf8().constData());
@@ -1294,7 +1369,7 @@ void TorrentHandle::handleStorageMovedAlert(libtorrent::storage_moved_alert *p)
 
     // Attempt to remove old folder if empty
     QDir oldSaveDir(Utils::Fs::fromNativePath(m_oldPath));
-    if ((oldSaveDir != QDir(m_session->defaultSavePath())) && (oldSaveDir != QDir(m_session->tempPath()))) {
+    if (oldSaveDir != QDir(m_session->defaultSavePath())) {
         qDebug("Attempting to remove %s", qPrintable(m_oldPath));
         QDir().rmpath(m_oldPath);
     }
@@ -1364,10 +1439,14 @@ void TorrentHandle::handleTorrentCheckedAlert(libtorrent::torrent_checked_alert
     qDebug("%s have just finished checking", qPrintable(hash()));
 
     updateStatus();
-    adjustActualSavePath();
 
-    if (progress() < 1.0 && wantedSize() > 0)
+    if ((progress() < 1.0) && (wantedSize() > 0))
         m_hasSeedStatus = false;
+    else if (progress() == 1.0)
+        m_hasSeedStatus = true;
+
+    adjustActualSavePath();
+    manageIncompleteFiles();
 
     if (m_pauseAfterRecheck) {
         m_pauseAfterRecheck = false;
@@ -1389,13 +1468,19 @@ void TorrentHandle::handleTorrentFinishedAlert(libtorrent::torrent_finished_aler
     m_hasSeedStatus = true;
 
     adjustActualSavePath();
-    if (Preferences::instance()->recheckTorrentsOnCompletion())
-        forceRecheck();
+    manageIncompleteFiles();
 
-    if (isMoveInProgress() || m_renameCount > 0)
+    const bool recheckTorrentsOnCompletion = Preferences::instance()->recheckTorrentsOnCompletion();
+    if (isMoveInProgress() || m_renameCount > 0) {
+        if (recheckTorrentsOnCompletion)
+            m_moveFinishedTriggers.append(boost::bind(&TorrentHandle::forceRecheck, this));
         m_moveFinishedTriggers.append(boost::bind(&Session::handleTorrentFinished, m_session, this));
-    else
+    }
+    else {
+        if (recheckTorrentsOnCompletion)
+            forceRecheck();
         m_session->handleTorrentFinished(this);
+    }
 }
 
 void TorrentHandle::handleTorrentPausedAlert(libtorrent::torrent_paused_alert *p)
@@ -1423,12 +1508,13 @@ void TorrentHandle::handleSaveResumeDataAlert(libtorrent::save_resume_data_alert
         resumeData["qBt-paused"] = isPaused();
         resumeData["qBt-forced"] = isForced();
     }
-    resumeData["qBt-savePath"] = Utils::String::toStdString(m_savePath);
+    resumeData["qBt-savePath"] = m_useAutoTMM ? "" : Utils::String::toStdString(m_savePath);
     resumeData["qBt-ratioLimit"] = Utils::String::toStdString(QString::number(m_ratioLimit));
-    resumeData["qBt-label"] = Utils::String::toStdString(m_label);
+    resumeData["qBt-category"] = Utils::String::toStdString(m_category);
     resumeData["qBt-name"] = Utils::String::toStdString(m_name);
     resumeData["qBt-seedStatus"] = m_hasSeedStatus;
     resumeData["qBt-tempPathDisabled"] = m_tempPathDisabled;
+    resumeData["qBt-queuePosition"] = queuePosition();
 
     m_session->handleTorrentResumeDataReady(this, resumeData);
 }
@@ -1485,12 +1571,6 @@ void TorrentHandle::handleFileRenamedAlert(libtorrent::file_renamed_alert *p)
 
     updateStatus();
 
-    if (filesCount() == 1) {
-        // Single-file torrent
-        // Renaming a file corresponds to changing the save path
-        m_session->handleTorrentSavePathChanged(this);
-    }
-
     --m_renameCount;
     while (!isMoveInProgress() && (m_renameCount == 0) && !m_moveFinishedTriggers.isEmpty())
         m_moveFinishedTriggers.takeFirst()();
@@ -1514,7 +1594,7 @@ void TorrentHandle::handleFileCompletedAlert(libtorrent::file_completed_alert *p
         QString name = filePath(p->index);
         if (name.endsWith(QB_EXT)) {
             const QString oldName = name;
-            name.chop(QString(QB_EXT).size());
+            name.chop(QB_EXT.size());
             qDebug("Renaming %s to %s", qPrintable(oldName), qPrintable(name));
             renameFile(p->index, name);
         }
@@ -1535,7 +1615,7 @@ void TorrentHandle::handleMetadataReceivedAlert(libt::metadata_received_alert *p
     qDebug("Metadata received for torrent %s.", qPrintable(name()));
     updateStatus();
     if (m_session->isAppendExtensionEnabled())
-        appendExtensionsToIncompleteFiles();
+        manageIncompleteFiles();
     m_session->handleTorrentMetadataReceived(this);
 
     if (isPaused()) {
@@ -1551,14 +1631,17 @@ void TorrentHandle::handleTempPathChanged()
     adjustActualSavePath();
 }
 
+void TorrentHandle::handleCategorySavePathChanged()
+{
+    if (m_useAutoTMM)
+        move_impl(m_session->categorySavePath(m_category));
+}
+
 void TorrentHandle::handleAppendExtensionToggled()
 {
     if (!hasMetadata()) return;
 
-    if (m_session->isAppendExtensionEnabled())
-        appendExtensionsToIncompleteFiles();
-    else
-        removeExtensionsFromIncompleteFiles();
+    manageIncompleteFiles();
 }
 
 void TorrentHandle::handleAlert(libtorrent::alert *a)
@@ -1615,30 +1698,26 @@ void TorrentHandle::handleAlert(libtorrent::alert *a)
     }
 }
 
-void TorrentHandle::appendExtensionsToIncompleteFiles()
+void TorrentHandle::manageIncompleteFiles()
 {
+    const bool isAppendExtensionEnabled = m_session->isAppendExtensionEnabled();
     QVector<qreal> fp = filesProgress();
     for (int i = 0; i < filesCount(); ++i) {
-        if ((fileSize(i) > 0) && (fp[i] < 1)) {
-            const QString name = filePath(i);
+        QString name = filePath(i);
+        if (isAppendExtensionEnabled && (fileSize(i) > 0) && (fp[i] < 1)) {
             if (!name.endsWith(QB_EXT)) {
                 const QString newName = name + QB_EXT;
-                qDebug("Renaming %s to %s", qPrintable(name), qPrintable(newName));
+                qDebug() << "Renaming" << name << "to" << newName;
                 renameFile(i, newName);
             }
         }
-    }
-}
-
-void TorrentHandle::removeExtensionsFromIncompleteFiles()
-{
-    for (int i = 0; i < filesCount(); ++i) {
-        QString name = filePath(i);
-        if (name.endsWith(QB_EXT)) {
-            const QString oldName = name;
-            name.chop(QString(QB_EXT).size());
-            qDebug("Renaming %s to %s", qPrintable(oldName), qPrintable(name));
-            renameFile(i, name);
+        else {
+            if (name.endsWith(QB_EXT)) {
+                const QString oldName = name;
+                name.chop(QB_EXT.size());
+                qDebug() << "Renaming" << oldName << "to" << name;
+                renameFile(i, name);
+            }
         }
     }
 }
@@ -1661,8 +1740,8 @@ void TorrentHandle::adjustActualSavePath_impl()
     }
     else {
         // Moving all downloading torrents to temporary save path
-        path = m_session->tempPath();
-        qDebug("Moving torrent to its temp save path: %s", qPrintable(path));
+        path = m_session->torrentTempPath(hash());
+        qDebug() << "Moving torrent to its temp save path:" << path;
     }
 
     moveStorage(Utils::Fs::toNativePath(path));
@@ -1676,15 +1755,11 @@ libtorrent::torrent_handle TorrentHandle::nativeHandle() const
 void TorrentHandle::updateTorrentInfo()
 {
     if (!hasMetadata()) return;
-
+#if LIBTORRENT_VERSION_NUM < 10100
     m_torrentInfo = TorrentInfo(m_nativeStatus.torrent_file);
-}
-
-void TorrentHandle::initialize()
-{
-    updateStatus();
-    m_hash = InfoHash(m_nativeStatus.info_hash);
-    adjustActualSavePath();
+#else
+    m_torrentInfo = TorrentInfo(m_nativeStatus.torrent_file.lock());
+#endif
 }
 
 bool TorrentHandle::isMoveInProgress() const
@@ -1694,7 +1769,7 @@ bool TorrentHandle::isMoveInProgress() const
 
 bool TorrentHandle::useTempPath() const
 {
-    return !m_tempPathDisabled && m_session->isTempPathEnabled() && !isSeed();
+    return !m_tempPathDisabled && m_session->isTempPathEnabled() && !(isSeed() || m_hasSeedStatus);
 }
 
 void TorrentHandle::updateStatus()
