@@ -30,15 +30,10 @@
  * Contact : hammered999@gmail.com
  */
 
-#include "preferences.h"
-#include "qinisettings.h"
-#include "logger.h"
-
 #include <QCryptographicHash>
 #include <QPair>
 #include <QDir>
-#include <QReadLocker>
-#include <QWriteLocker>
+#include <QSettings>
 
 #ifndef DISABLE_GUI
 #include <QApplication>
@@ -47,79 +42,27 @@
 #endif
 
 #ifdef Q_OS_WIN
-#include <ShlObj.h>
+#include <shlobj.h>
 #include <winreg.h>
 #endif
 
-#include <cstdlib>
-#include "base/utils/fs.h"
-#include "base/utils/misc.h"
+#ifdef Q_OS_MAC
+#include <CoreServices/CoreServices.h>
+#endif
 
+#include <cstdlib>
+
+#include "utils/fs.h"
+#include "utils/misc.h"
+#include "settingsstorage.h"
+#include "logger.h"
+#include "preferences.h"
 
 Preferences* Preferences::m_instance = 0;
 
 Preferences::Preferences()
     : m_randomPort(rand() % 64512 + 1024)
-    , dirty(false)
-    , lock(QReadWriteLock::Recursive)
 {
-    qRegisterMetaTypeStreamOperators<MaxRatioAction>("MaxRatioAction");
-
-    QIniSettings *settings = new QIniSettings;
-#ifndef Q_OS_MAC
-    QIniSettings *settings_new = new QIniSettings("qBittorrent", "qBittorrent_new");
-    QStringList keys = settings_new->allKeys();
-    bool use_new = false;
-
-    // This means that the PC closed either due to power outage
-    // or because the disk was full. In any case the settings weren't transfered
-    // in their final position. So assume that qbittorrent_new.ini/qbittorrent_new.conf
-    // contains the most recent settings.
-    if (!keys.isEmpty()) {
-        Logger::instance()->addMessage(tr("Detected unclean program exit. Using fallback file to restore settings."), Log::WARNING);
-        use_new = true;
-        dirty = true;
-    }
-    else {
-        keys = settings->allKeys();
-    }
-#else
-    QStringList keys = settings->allKeys();
-#endif
-
-    // Copy everything into memory. This means even keys inserted in the file manually
-    // or that we don't touch directly in this code(eg disabled by ifdef). This ensures
-    // that they will be copied over when save our settings to disk.
-    for (QStringList::const_iterator i = keys.begin(), e = keys.end(); i != e; ++i) {
-#ifndef Q_OS_MAC
-        if (!use_new)
-            m_data[*i] = settings->value(*i);
-        else
-            m_data[*i] = settings_new->value(*i);
-#else
-        m_data[*i] = settings->value(*i);
-#endif
-    }
-
-    //Ensures sync to disk before we attempt to manipulate the files from save().
-    delete settings;
-#ifndef Q_OS_MAC
-    QString new_path = settings_new->fileName();
-    delete settings_new;
-    Utils::Fs::forceRemove(new_path);
-
-    if (use_new)
-        save();
-#endif
-
-    timer.setSingleShot(true);
-    timer.setInterval(5 * 1000);
-    connect(&timer, SIGNAL(timeout()), SLOT(save()));
-}
-
-Preferences::~Preferences()
-{
-    save();
 }
 
 Preferences *Preferences::instance()
@@ -141,71 +84,14 @@ void Preferences::freeInstance()
     }
 }
 
-bool Preferences::save()
-{
-    QWriteLocker locker(&lock);
-
-    if (!dirty) return false;
-
-#ifndef Q_OS_MAC
-    // QSettings delete the file before writing it out. This can result in problems
-    // if the disk is full or a power outage occurs. Those events might occur
-    // between deleting the file and recreating it. This is a safety measure.
-    // Write everything to qBittorrent_new.ini/qBittorrent_new.conf and if it succeeds
-    // replace qBittorrent_new.ini/qBittorrent.conf with it.
-    QIniSettings *settings = new QIniSettings("qBittorrent", "qBittorrent_new");
-#else
-    QIniSettings *settings = new QIniSettings;
-#endif
-
-    for (QHash<QString, QVariant>::const_iterator i = m_data.begin(), e = m_data.end(); i != e; ++i)
-        settings->setValue(i.key(), i.value());
-
-    dirty = false;
-    locker.unlock();
-
-#ifndef Q_OS_MAC
-    settings->sync(); // Important to get error status
-    QString new_path = settings->fileName();
-    QSettings::Status status = settings->status();
-
-    if (status != QSettings::NoError) {
-        if (status == QSettings::AccessError)
-            Logger::instance()->addMessage(tr("An access error occurred while trying to write the configuration file."), Log::CRITICAL);
-        else
-            Logger::instance()->addMessage(tr("A format error occurred while trying to write the configuration file."), Log::CRITICAL);
-
-        delete settings;
-        Utils::Fs::forceRemove(new_path);
-        return false;
-    }
-    delete settings;
-    QString final_path = new_path;
-    int index = final_path.lastIndexOf("_new", -1, Qt::CaseInsensitive);
-    final_path.remove(index, 4);
-    Utils::Fs::forceRemove(final_path);
-    QFile::rename(new_path, final_path);
-#else
-    delete settings;
-#endif
-
-    return true;
-}
-
 const QVariant Preferences::value(const QString &key, const QVariant &defaultValue) const
 {
-    QReadLocker locker(&lock);
-    return m_data.value(key, defaultValue);
+    return SettingsStorage::instance()->loadValue(key, defaultValue);
 }
 
 void Preferences::setValue(const QString &key, const QVariant &value)
 {
-    QWriteLocker locker(&lock);
-    if (m_data.value(key) == value)
-        return;
-    dirty = true;
-    timer.start();
-    m_data.insert(key, value);
+    SettingsStorage::instance()->storeValue(key, value);
 }
 
 // General options
@@ -219,16 +105,6 @@ void Preferences::setLocale(const QString &locale)
     setValue("Preferences/General/Locale", locale);
 }
 
-bool Preferences::useProgramNotification() const
-{
-    return value("Preferences/General/ProgramNotification", true).toBool();
-}
-
-void Preferences::useProgramNotification(bool use)
-{
-    setValue("Preferences/General/ProgramNotification", use);
-}
-
 bool Preferences::deleteTorrentFilesAsDefault() const
 {
     return value("Preferences/General/DeleteTorrentsFilesAsDefault", false).toBool();
@@ -269,6 +145,26 @@ void Preferences::setAlternatingRowColors(bool b)
     setValue("Preferences/General/AlternatingRowColors", b);
 }
 
+bool Preferences::getHideZeroValues() const
+{
+    return value("Preferences/General/HideZeroValues", false).toBool();
+}
+
+void Preferences::setHideZeroValues(bool b)
+{
+    setValue("Preferences/General/HideZeroValues", b);
+}
+
+int Preferences::getHideZeroComboValues() const
+{
+    return value("Preferences/General/HideZeroComboValues", 0).toInt();
+}
+
+void Preferences::setHideZeroComboValues(int n)
+{
+    setValue("Preferences/General/HideZeroComboValues", n);
+}
+
 bool Preferences::useRandomPort() const
 {
     return value("Preferences/General/UseRandomPort", false).toBool();
@@ -331,7 +227,7 @@ void Preferences::setStartMinimized(bool b)
 
 bool Preferences::isSplashScreenDisabled() const
 {
-    return value("Preferences/General/NoSplashScreen", false).toBool();
+    return value("Preferences/General/NoSplashScreen", true).toBool();
 }
 
 void Preferences::setSplashScreenDisabled(bool b)
@@ -371,40 +267,6 @@ void Preferences::setWinStartup(bool b)
 #endif
 
 // Downloads
-QString Preferences::getSavePath() const
-{
-    QString save_path = value("Preferences/Downloads/SavePath").toString();
-    if (!save_path.isEmpty())
-        return Utils::Fs::fromNativePath(save_path);
-    return Utils::Fs::QDesktopServicesDownloadLocation();
-}
-
-void Preferences::setSavePath(const QString &save_path)
-{
-    setValue("Preferences/Downloads/SavePath", Utils::Fs::fromNativePath(save_path));
-}
-
-bool Preferences::isTempPathEnabled() const
-{
-    return value("Preferences/Downloads/TempPathEnabled", false).toBool();
-}
-
-void Preferences::setTempPathEnabled(bool enabled)
-{
-    setValue("Preferences/Downloads/TempPathEnabled", enabled);
-}
-
-QString Preferences::getTempPath() const
-{
-    const QString temp = QDir(getSavePath()).absoluteFilePath("temp");
-    return Utils::Fs::fromNativePath(value("Preferences/Downloads/TempPath", temp).toString());
-}
-
-void Preferences::setTempPath(const QString &path)
-{
-    setValue("Preferences/Downloads/TempPath", Utils::Fs::fromNativePath(path));
-}
-
 bool Preferences::useIncompleteFilesExtension() const
 {
     return value("Preferences/Downloads/UseIncompleteExtension", false).toBool();
@@ -415,16 +277,6 @@ void Preferences::useIncompleteFilesExtension(bool enabled)
     setValue("Preferences/Downloads/UseIncompleteExtension", enabled);
 }
 
-bool Preferences::appendTorrentLabel() const
-{
-    return value("Preferences/Downloads/AppendLabel", false).toBool();
-}
-
-void Preferences::setAppendTorrentLabel(bool b)
-{
-    setValue("Preferences/Downloads/AppendLabel", b);
-}
-
 QString Preferences::lastLocationPath() const
 {
     return Utils::Fs::fromNativePath(value("Preferences/Downloads/LastLocationPath").toString());
@@ -445,76 +297,15 @@ void Preferences::preAllocateAllFiles(bool enabled)
     return setValue("Preferences/Downloads/PreAllocation", enabled);
 }
 
-bool Preferences::useAdditionDialog() const
+QVariantHash Preferences::getScanDirs() const
 {
-    return value("Preferences/Downloads/NewAdditionDialog", true).toBool();
-}
-
-void Preferences::useAdditionDialog(bool b)
-{
-    setValue("Preferences/Downloads/NewAdditionDialog", b);
-}
-
-bool Preferences::additionDialogFront() const
-{
-    return value("Preferences/Downloads/NewAdditionDialogFront", true).toBool();
-}
-
-void Preferences::additionDialogFront(bool b)
-{
-    setValue("Preferences/Downloads/NewAdditionDialogFront", b);
-}
-
-bool Preferences::addTorrentsInPause() const
-{
-    return value("Preferences/Downloads/StartInPause", false).toBool();
-}
-
-void Preferences::addTorrentsInPause(bool b)
-{
-    setValue("Preferences/Downloads/StartInPause", b);
-}
-
-QStringList Preferences::getScanDirs() const
-{
-    QStringList originalList = value("Preferences/Downloads/ScanDirs").toStringList();
-    if (originalList.isEmpty())
-        return originalList;
-
-    QStringList newList;
-    foreach (const QString& s, originalList)
-        newList << Utils::Fs::fromNativePath(s);
-    return newList;
+    return value("Preferences/Downloads/ScanDirsV2").toHash();
 }
 
 // This must be called somewhere with data from the model
-void Preferences::setScanDirs(const QStringList &dirs)
+void Preferences::setScanDirs(const QVariantHash &dirs)
 {
-    QStringList newList;
-    if (!dirs.isEmpty())
-        foreach (const QString& s, dirs)
-            newList << Utils::Fs::fromNativePath(s);
-    setValue("Preferences/Downloads/ScanDirs", newList);
-}
-
-QList<bool> Preferences::getDownloadInScanDirs() const
-{
-    return Utils::Misc::boolListfromStringList(value("Preferences/Downloads/DownloadInScanDirs").toStringList());
-}
-
-void Preferences::setDownloadInScanDirs(const QList<bool> &list)
-{
-    setValue("Preferences/Downloads/DownloadInScanDirs", Utils::Misc::toStringList(list));
-}
-
-void Preferences::setScanDirsDownloadPaths(const QStringList &downloadpaths)
-{
-    setValue("Preferences/Downloads/ScanDirsDownloadPaths", downloadpaths);
-}
-
-QStringList Preferences::getScanDirsDownloadPaths() const
-{
-    return value("Preferences/Downloads/ScanDirsDownloadPaths").toStringList();
+    setValue("Preferences/Downloads/ScanDirsV2", dirs);
 }
 
 QString Preferences::getScanDirsLastPath() const
@@ -650,7 +441,6 @@ void Preferences::setActionOnDblClOnTorrentFn(int act)
 // Connection options
 int Preferences::getSessionPort() const
 {
-    QReadLocker locker(&lock);
     if (useRandomPort())
         return m_randomPort;
     return value("Preferences/Connection/PortRangeMin", 8999).toInt();
@@ -996,7 +786,7 @@ void Preferences::setTrackersList(const QString &val)
 
 qreal Preferences::getGlobalMaxRatio() const
 {
-    return value("Preferences/Bittorrent/MaxRatio", -1).toDouble();
+    return value("Preferences/Bittorrent/MaxRatio", -1).toReal();
 }
 
 void Preferences::setGlobalMaxRatio(qreal ratio)
@@ -1004,16 +794,6 @@ void Preferences::setGlobalMaxRatio(qreal ratio)
     setValue("Preferences/Bittorrent/MaxRatio", ratio);
 }
 
-MaxRatioAction Preferences::getMaxRatioAction() const
-{
-    return value("Preferences/Bittorrent/MaxRatioAction", QVariant::fromValue(MaxRatioAction::Pause)).value<MaxRatioAction>();
-}
-
-void Preferences::setMaxRatioAction(MaxRatioAction act)
-{
-    setValue("Preferences/Bittorrent/MaxRatioAction", QVariant::fromValue(act));
-}
-
 // IP Filter
 bool Preferences::isFilteringEnabled() const
 {
@@ -1027,12 +807,12 @@ void Preferences::setFilteringEnabled(bool enabled)
 
 bool Preferences::isFilteringTrackerEnabled() const
 {
-	return value("Preferences/IPFilter/FilterTracker", false).toBool();
+    return value("Preferences/IPFilter/FilterTracker", false).toBool();
 }
 
 void Preferences::setFilteringTrackerEnabled(bool enabled)
 {
-	setValue("Preferences/IPFilter/FilterTracker", enabled);
+    setValue("Preferences/IPFilter/FilterTracker", enabled);
 }
 
 QString Preferences::getFilter() const
@@ -1070,17 +850,6 @@ void Preferences::setSearchEnabled(bool enabled)
     setValue("Preferences/Search/SearchEnabled", enabled);
 }
 
-// Execution Log
-bool Preferences::isExecutionLogEnabled() const
-{
-    return value("Preferences/ExecutionLog/enabled", false).toBool();
-}
-
-void Preferences::setExecutionLogEnabled(bool b)
-{
-    setValue("Preferences/ExecutionLog/enabled", b);
-}
-
 // Queueing system
 bool Preferences::isQueueingSystemEnabled() const
 {
@@ -1337,12 +1106,12 @@ void Preferences::setAutoRunEnabled(bool enabled)
 
 QString Preferences::getAutoRunProgram() const
 {
-    return Utils::Fs::fromNativePath(value("AutoRun/program").toString());
+    return value("AutoRun/program").toString();
 }
 
 void Preferences::setAutoRunProgram(const QString &program)
 {
-    setValue("AutoRun/program", Utils::Fs::fromNativePath(program));
+    setValue("AutoRun/program", program);
 }
 
 bool Preferences::shutdownWhenDownloadsComplete() const
@@ -1385,6 +1154,16 @@ void Preferences::setShutdownqBTWhenDownloadsComplete(bool shutdown)
     setValue("Preferences/Downloads/AutoShutDownqBTOnCompletion", shutdown);
 }
 
+bool Preferences::dontConfirmAutoExit() const
+{
+    return value("ShutdownConfirmDlg/DontConfirmAutoExit", false).toBool();
+}
+
+void Preferences::setDontConfirmAutoExit(bool dontConfirmAutoExit)
+{
+    setValue("ShutdownConfirmDlg/DontConfirmAutoExit", dontConfirmAutoExit);
+}
+
 uint Preferences::diskCacheSize() const
 {
     uint size = value("Preferences/Downloads/DiskWriteCacheSize", 0).toUInt();
@@ -1565,6 +1344,16 @@ void Preferences::setNetworkInterfaceName(const QString& iface)
     setValue("Preferences/Connection/InterfaceName", iface);
 }
 
+void Preferences::setNetworkInterfaceAddress(const QString& addr)
+{
+    setValue("Preferences/Connection/InterfaceAddress", addr);
+}
+
+QString Preferences::getNetworkInterfaceAddress() const
+{
+    return value("Preferences/Connection/InterfaceAddress").toString();
+}
+
 bool Preferences::getListenIPv6() const
 {
     return value("Preferences/Connection/InterfaceListenIPv6", false).toBool();
@@ -1627,51 +1416,6 @@ void Preferences::useSystemIconTheme(bool enabled)
 }
 #endif
 
-QStringList Preferences::getTorrentLabels() const
-{
-    return value("TransferListFilters/customLabels").toStringList();
-}
-
-void Preferences::setTorrentLabels(const QStringList& labels)
-{
-    setValue("TransferListFilters/customLabels", labels);
-}
-
-void Preferences::addTorrentLabelExternal(const QString &label)
-{
-    addTorrentLabel(label);
-    QString toEmit = label;
-    emit externalLabelAdded(toEmit);
-}
-
-void Preferences::addTorrentLabel(const QString& label)
-{
-    QStringList labels = value("TransferListFilters/customLabels").toStringList();
-    if (labels.contains(label))
-        return;
-    labels << label;
-    setValue("TransferListFilters/customLabels", labels);
-}
-
-void Preferences::removeTorrentLabel(const QString& label)
-{
-    QStringList labels = value("TransferListFilters/customLabels").toStringList();
-    if (!labels.contains(label))
-        return;
-    labels.removeOne(label);
-    setValue("TransferListFilters/customLabels", labels);
-}
-
-QString Preferences::getDefaultLabel() const
-{
-    return value("Preferences/Downloads/DefaultLabel").toString();
-}
-
-void Preferences::setDefaultLabel(const QString &defaultLabel)
-{
-    setValue("Preferences/Downloads/DefaultLabel", defaultLabel);
-}
-
 bool Preferences::recursiveDownloadDisabled() const
 {
     return value("Preferences/Advanced/DisableRecursiveDownload", false).toBool();
@@ -1910,6 +1654,62 @@ void Preferences::setMagnetLinkAssoc(bool set)
 }
 #endif
 
+#ifdef Q_OS_MAC
+namespace
+{
+    CFStringRef torrentExtension = CFSTR("torrent");
+    CFStringRef magnetUrlScheme = CFSTR("magnet");
+}
+
+bool Preferences::isTorrentFileAssocSet()
+{
+    bool isSet = false;
+    CFStringRef torrentId = UTTypeCreatePreferredIdentifierForTag(kUTTagClassFilenameExtension, torrentExtension, NULL);
+    if (torrentId != NULL) {
+        CFStringRef defaultHandlerId = LSCopyDefaultRoleHandlerForContentType(torrentId, kLSRolesViewer);
+        if (defaultHandlerId != NULL) {
+            CFStringRef myBundleId = CFBundleGetIdentifier(CFBundleGetMainBundle());
+            isSet = CFStringCompare(myBundleId, defaultHandlerId, 0) == kCFCompareEqualTo;
+            CFRelease(defaultHandlerId);
+        }
+        CFRelease(torrentId);
+    }
+    return isSet;
+}
+
+bool Preferences::isMagnetLinkAssocSet()
+{
+    bool isSet = false;
+    CFStringRef defaultHandlerId = LSCopyDefaultHandlerForURLScheme(magnetUrlScheme);
+    if (defaultHandlerId != NULL) {
+        CFStringRef myBundleId = CFBundleGetIdentifier(CFBundleGetMainBundle());
+        isSet = CFStringCompare(myBundleId, defaultHandlerId, 0) == kCFCompareEqualTo;
+        CFRelease(defaultHandlerId);
+    }
+    return isSet;
+}
+
+void Preferences::setTorrentFileAssoc()
+{
+    if (isTorrentFileAssocSet())
+        return;
+    CFStringRef torrentId = UTTypeCreatePreferredIdentifierForTag(kUTTagClassFilenameExtension, torrentExtension, NULL);
+    if (torrentId != NULL) {
+        CFStringRef myBundleId = CFBundleGetIdentifier(CFBundleGetMainBundle());
+        LSSetDefaultRoleHandlerForContentType(torrentId, kLSRolesViewer, myBundleId);
+        CFRelease(torrentId);
+    }
+}
+
+void Preferences::setMagnetLinkAssoc()
+{
+    if (isMagnetLinkAssocSet())
+        return;
+    CFStringRef myBundleId = CFBundleGetIdentifier(CFBundleGetMainBundle());
+    LSSetDefaultHandlerForURLScheme(magnetUrlScheme, myBundleId);
+}
+#endif
+
 bool Preferences::isTrackerEnabled() const
 {
     return value("Preferences/Advanced/trackerEnabled", false).toBool();
@@ -1974,63 +1774,6 @@ void Preferences::setTrayIconStyle(TrayIcon::Style style)
 
 // Stuff that don't appear in the Options GUI but are saved
 // in the same file.
-QByteArray Preferences::getAddNewTorrentDialogState() const
-{
-#ifdef QBT_USES_QT5
-    return value("AddNewTorrentDialog/qt5/treeHeaderState").toByteArray();
-#else
-    return value("AddNewTorrentDialog/treeHeaderState").toByteArray();
-#endif
-}
-
-void Preferences::setAddNewTorrentDialogState(const QByteArray &state)
-{
-#ifdef QBT_USES_QT5
-    setValue("AddNewTorrentDialog/qt5/treeHeaderState", state);
-#else
-    setValue("AddNewTorrentDialog/treeHeaderState", state);
-#endif
-}
-
-int Preferences::getAddNewTorrentDialogPos() const
-{
-    return value("AddNewTorrentDialog/y", -1).toInt();
-}
-
-void Preferences::setAddNewTorrentDialogPos(const int &pos)
-{
-    setValue("AddNewTorrentDialog/y", pos);
-}
-
-int Preferences::getAddNewTorrentDialogWidth() const
-{
-    return value("AddNewTorrentDialog/width", -1).toInt();
-}
-
-void Preferences::setAddNewTorrentDialogWidth(const int &width)
-{
-    setValue("AddNewTorrentDialog/width", width);
-}
-
-bool Preferences::getAddNewTorrentDialogExpanded() const
-{
-    return value("AddNewTorrentDialog/expanded", false).toBool();
-}
-
-void Preferences::setAddNewTorrentDialogExpanded(const bool expanded)
-{
-    setValue("AddNewTorrentDialog/expanded", expanded);
-}
-
-QStringList Preferences::getAddNewTorrentDialogPathHistory() const
-{
-    return value("TorrentAdditionDlg/save_path_history").toStringList();
-}
-
-void Preferences::setAddNewTorrentDialogPathHistory(const QStringList &history)
-{
-    setValue("TorrentAdditionDlg/save_path_history", history);
-}
 
 QDateTime Preferences::getDNSLastUpd() const
 {
@@ -2390,14 +2133,14 @@ void Preferences::setStatusFilterState(const bool checked)
     setValue("TransferListFilters/statusFilterState", checked);
 }
 
-bool Preferences::getLabelFilterState() const
+bool Preferences::getCategoryFilterState() const
 {
-    return value("TransferListFilters/labelFilterState", true).toBool();
+    return value("TransferListFilters/CategoryFilterState", true).toBool();
 }
 
-void Preferences::setLabelFilterState(const bool checked)
+void Preferences::setCategoryFilterState(const bool checked)
 {
-    setValue("TransferListFilters/labelFilterState", checked);
+    setValue("TransferListFilters/CategoryFilterState", checked);
 }
 
 bool Preferences::getTrackerFilterState() const
@@ -2438,22 +2181,6 @@ void Preferences::setTransHeaderState(const QByteArray &state)
 #endif
 }
 
-// Temp code.
-// See TorrentStatistics::loadStats() for details.
-QVariantHash Preferences::getStats() const
-{
-    return value("Stats/AllStats").toHash();
-}
-
-void Preferences::removeStats()
-{
-    QWriteLocker locker(&lock);
-    dirty = true;
-    if (!timer.isActive())
-        timer.start();
-    m_data.remove("Stats/AllStats");
-}
-
 //From old RssSettings class
 bool Preferences::isRSSEnabled() const
 {
@@ -2467,7 +2194,7 @@ void Preferences::setRSSEnabled(const bool enabled)
 
 uint Preferences::getRSSRefreshInterval() const
 {
-    return value("Preferences/RSS/RSSRefresh", 5).toUInt();
+    return value("Preferences/RSS/RSSRefresh", 30).toUInt();
 }
 
 void Preferences::setRSSRefreshInterval(const uint &interval)
@@ -2525,45 +2252,32 @@ void Preferences::setToolbarTextPosition(const int position)
     setValue("Toolbar/textPosition", position);
 }
 
-QList<QByteArray> Preferences::getHostNameCookies(const QString &host_name) const
-{
-    QMap<QString, QVariant> hosts_table = value("Rss/hosts_cookies").toMap();
-    if (!hosts_table.contains(host_name)) return QList<QByteArray>();
-    QByteArray raw_cookies = hosts_table.value(host_name).toByteArray();
-    return raw_cookies.split(':');
-}
-
-QList<QNetworkCookie> Preferences::getHostNameQNetworkCookies(const QString& host_name) const
+QList<QNetworkCookie> Preferences::getNetworkCookies() const
 {
     QList<QNetworkCookie> cookies;
-    const QList<QByteArray> raw_cookies = getHostNameCookies(host_name);
-    foreach (const QByteArray& raw_cookie, raw_cookies) {
-        QList<QByteArray> cookie_parts = raw_cookie.split('=');
-        if (cookie_parts.size() == 2) {
-            qDebug("Loading cookie: %s = %s", cookie_parts.first().constData(), cookie_parts.last().constData());
-            cookies << QNetworkCookie(cookie_parts.first(), cookie_parts.last());
-        }
-    }
+    QStringList rawCookies = value("Network/Cookies").toStringList();
+    foreach (const QString &rawCookie, rawCookies)
+        cookies << QNetworkCookie::parseCookies(rawCookie.toUtf8());
+
     return cookies;
 }
 
-void Preferences::setHostNameCookies(const QString &host_name, const QList<QByteArray> &cookies)
+void Preferences::setNetworkCookies(const QList<QNetworkCookie> &cookies)
 {
-    QMap<QString, QVariant> hosts_table = value("Rss/hosts_cookies").toMap();
-    QByteArray raw_cookies = "";
-    foreach (const QByteArray& cookie, cookies)
-        raw_cookies += cookie + ":";
-    if (raw_cookies.endsWith(":"))
-        raw_cookies.chop(1);
-    hosts_table.insert(host_name, raw_cookies);
-    setValue("Rss/hosts_cookies", hosts_table);
+    QStringList rawCookies;
+    foreach (const QNetworkCookie &cookie, cookies)
+        rawCookies << cookie.toRawForm();
+
+    setValue("Network/Cookies", rawCookies);
 }
 
-int Preferences::getSpeedWidgetPeriod() const {
+int Preferences::getSpeedWidgetPeriod() const
+{
     return value("SpeedWidget/period", 1).toInt();
 }
 
-void Preferences::setSpeedWidgetPeriod(const int period) {
+void Preferences::setSpeedWidgetPeriod(const int period)
+{
     setValue("SpeedWidget/period", period);
 }
 
@@ -2578,8 +2292,43 @@ void Preferences::setSpeedWidgetGraphEnable(int id, const bool enable)
     setValue("SpeedWidget/graph_enable_" + QString::number(id), enable);
 }
 
+void Preferences::upgrade()
+{
+    // Move RSS cookies to global storage
+    QList<QNetworkCookie> cookies = getNetworkCookies();
+    QVariantMap hostsTable = value("Rss/hosts_cookies").toMap();
+    foreach (const QString &key, hostsTable.keys()) {
+        QVariant value = hostsTable[key];
+        QList<QByteArray> rawCookies = value.toByteArray().split(':');
+        foreach (const QByteArray &rawCookie, rawCookies) {
+            foreach (QNetworkCookie cookie, QNetworkCookie::parseCookies(rawCookie)) {
+                cookie.setDomain(key);
+                cookie.setPath("/");
+                cookie.setExpirationDate(QDateTime::currentDateTime().addYears(10));
+                cookies << cookie;
+            }
+        }
+    }
+
+    setNetworkCookies(cookies);
+
+    QStringList labels = value("TransferListFilters/customLabels").toStringList();
+    if (!labels.isEmpty()) {
+        QVariantMap categories = value("BitTorrent/Session/Categories").toMap();
+        foreach (const QString &label, labels) {
+            if (!categories.contains(label))
+                categories[label] = "";
+        }
+        setValue("BitTorrent/Session/Categories", categories);
+        SettingsStorage::instance()->removeValue("TransferListFilters/customLabels");
+    }
+
+    SettingsStorage::instance()->removeValue("Rss/hosts_cookies");
+    SettingsStorage::instance()->removeValue("Preferences/Downloads/AppendLabel");
+}
+
 void Preferences::apply()
 {
-    if (save())
+    if (SettingsStorage::instance()->save())
         emit changed();
 }
