@@ -31,20 +31,26 @@
 #ifndef ADDNEWTORRENTDIALOG_H
 #define ADDNEWTORRENTDIALOG_H
 
-#include <QShortcut>
 #include <QDialog>
+#include <QScopedPointer>
+#include <QShortcut>
 #include <QUrl>
 
 #include "base/bittorrent/infohash.h"
 #include "base/bittorrent/torrentinfo.h"
 
-QT_BEGIN_NAMESPACE
-namespace Ui {
+namespace BitTorrent
+{
+    class MagnetUri;
+}
+
+namespace Ui
+{
     class AddNewTorrentDialog;
 }
-QT_END_NAMESPACE
 
 class TorrentContentFilterModel;
+class TorrentFileGuard;
 class PropListDelegate;
 
 class AddNewTorrentDialog: public QDialog
@@ -54,17 +60,18 @@ class AddNewTorrentDialog: public QDialog
 public:
     ~AddNewTorrentDialog();
 
-    static void show(QString source, QWidget *parent = 0);
+    static bool isEnabled();
+    static void setEnabled(bool value);
+    static bool isTopLevel();
+    static void setTopLevel(bool value);
 
-protected:
-    void showEvent(QShowEvent *event);
+    static void show(QString source, QWidget *parent = 0);
 
 private slots:
     void showAdvancedSettings(bool show);
     void displayContentTreeMenu(const QPoint&);
     void updateDiskSpaceLabel();
     void onSavePathChanged(int);
-    void relayout();
     void renameSelectedFile();
     void setdialogPosition();
     void updateMetadata(const BitTorrent::TorrentInfo &info);
@@ -72,25 +79,29 @@ private slots:
     void handleDownloadFailed(const QString &url, const QString &reason);
     void handleRedirectedToMagnet(const QString &url, const QString &magnetUri);
     void handleDownloadFinished(const QString &url, const QString &filePath);
+    void TMMChanged(int index);
+    void categoryChanged(int index);
+    void doNotDeleteTorrentClicked(bool checked);
 
-protected slots:
-    virtual void accept();
-    virtual void reject();
+    void accept() override;
+    void reject() override;
 
 private:
     explicit AddNewTorrentDialog(QWidget *parent = 0);
-    bool loadTorrent(const QString& torrent_path);
-    bool loadMagnet(const QString& magnet_uri);
-    void loadSavePathHistory();
+    bool loadTorrent(const QString &torrentPath);
+    bool loadMagnet(const BitTorrent::MagnetUri &magnetUri);
+    void populateSavePathComboBox();
     void saveSavePathHistory() const;
     int indexOfSavePath(const QString& save_path);
-    void updateFileNameInSavePaths(const QString& new_filename);
     void loadState();
     void saveState();
     void setMetadataProgressIndicator(bool visibleIndicator, const QString &labelText = QString());
     void setupTreeview();
+    QString defaultSavePath() const;
+    void setCommentText(const QString &str) const;
+
+    void showEvent(QShowEvent *event) override;
 
-private:
     Ui::AddNewTorrentDialog *ui;
     TorrentContentFilterModel *m_contentModel;
     PropListDelegate *m_contentDelegate;
@@ -101,6 +112,7 @@ private:
     QShortcut *editHotkey;
     QByteArray m_headerState;
     int m_oldIndex;
+    QScopedPointer<TorrentFileGuard> m_torrentGuard;
 };
 
 #endif // ADDNEWTORRENTDIALOG_H
