@@ -42,7 +42,7 @@ class DeletionConfirmationDlg : public QDialog, private Ui::confirmDeletionDlg {
   Q_OBJECT
 
   public:
-  DeletionConfirmationDlg(QWidget *parent, const int &size, const QString &name): QDialog(parent) {
+  DeletionConfirmationDlg(QWidget *parent, const int &size, const QString &name, bool defaultDeleteFiles): QDialog(parent) {
     setupUi(this);
     if (size == 1)
       label->setText(tr("Are you sure you want to delete '%1' from the transfer list?", "Are you sure you want to delete 'ubuntu-linux-iso' from the transfer list?").arg(name));
@@ -54,7 +54,7 @@ class DeletionConfirmationDlg : public QDialog, private Ui::confirmDeletionDlg {
     rememberBtn->setIcon(GuiIconProvider::instance()->getIcon("object-locked"));
 
     move(Utils::Misc::screenCenter(this));
-    checkPermDelete->setChecked(Preferences::instance()->deleteTorrentFilesAsDefault());
+    checkPermDelete->setChecked(defaultDeleteFiles || Preferences::instance()->deleteTorrentFilesAsDefault());
     connect(checkPermDelete, SIGNAL(clicked()), this, SLOT(updateRememberButtonState()));
     buttonBox->button(QDialogButtonBox::Cancel)->setFocus();
   }
@@ -63,10 +63,10 @@ class DeletionConfirmationDlg : public QDialog, private Ui::confirmDeletionDlg {
     return checkPermDelete->isChecked();
   }
 
-  static bool askForDeletionConfirmation(bool& delete_local_files, const int& size, const QString& name) {
-    DeletionConfirmationDlg dlg(NULL, size, name);
+  static bool askForDeletionConfirmation(bool& deleteLocalFiles, const int& size, const QString& name) {
+    DeletionConfirmationDlg dlg(NULL, size, name, deleteLocalFiles);
     if (dlg.exec() == QDialog::Accepted) {
-      delete_local_files = dlg.shouldDeleteLocalFiles();
+      deleteLocalFiles = dlg.shouldDeleteLocalFiles();
       return true;
     }
     return false;
