@@ -68,9 +68,8 @@ public:
     ~PeerListWidget();
 
     void loadPeers(BitTorrent::TorrentHandle *const torrent, bool forceHostnameResolution = false);
-    QStandardItem *addPeer(const QString &ip, const BitTorrent::PeerInfo &peer);
-    void updatePeer(const QString &ip, const BitTorrent::PeerInfo &peer);
-    void handleResolved(const QString &ip, const QString &hostname);
+    QStandardItem *addPeer(const QString &ip, BitTorrent::TorrentHandle *const torrent, const BitTorrent::PeerInfo &peer);
+    void updatePeer(const QString &ip, BitTorrent::TorrentHandle *const torrent, const BitTorrent::PeerInfo &peer);
     void updatePeerHostNameResolutionState();
     void updatePeerCountryResolutionState();
     void clear();
@@ -78,10 +77,12 @@ public:
 private slots:
     void loadSettings();
     void saveSettings() const;
+    void displayToggleColumnsMenu(const QPoint&);
     void showPeerListMenu(const QPoint&);
     void banSelectedPeers();
     void copySelectedPeers();
     void handleSortColumnChanged(int col);
+    void handleResolved(const QString &ip, const QString &hostname);
 
 private:
     QStandardItemModel *m_listModel;
@@ -92,7 +93,7 @@ private:
     QSet<QString> m_missingFlags;
     QPointer<Net::ReverseResolution> m_resolver;
     PropertiesWidget *m_properties;
-    bool m_displayFlags;
+    bool m_resolveCountries;
     QShortcut *m_copyHotkey;
 };
 
