@@ -41,21 +41,18 @@ QT_END_NAMESPACE
 
 // Defines for download list list columns
 
-class TransferListDelegate: public QItemDelegate {
-  Q_OBJECT
+class TransferListDelegate: public QItemDelegate
+{
+    Q_OBJECT
 
 public:
-  TransferListDelegate(QObject *parent);
-  ~TransferListDelegate();
-  void paint(QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index) const;
-  QWidget* createEditor(QWidget*, const QStyleOptionViewItem &, const QModelIndex &) const;
-
-  // Reimplementing sizeHint() because the 'name' column contains text+icon.
-  // When that WHOLE column goes out of view(eg user scrolls horizontally)
-  // the rows shrink if the text's height is smaller than the icon's height.
-  // This happens because icon from the 'name' column is no longer drawn.
-  QSize sizeHint(const QStyleOptionViewItem & option, const QModelIndex & index) const;
+    TransferListDelegate(QObject *parent);
+    void paint(QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index) const;
+    QWidget* createEditor(QWidget*, const QStyleOptionViewItem &, const QModelIndex &) const;
+    QSize sizeHint(const QStyleOptionViewItem & option, const QModelIndex & index) const;
 
+private:
+    QString getStatusString(const int state) const;
 };
 
 #endif // TRANSFERLISTDELEGATE_H
