@@ -90,22 +90,22 @@ private:
     virtual void torrentAboutToBeDeleted(BitTorrent::TorrentHandle *const);
 };
 
-class LabelFiltersList: public FiltersBase
+class CategoryFiltersList: public FiltersBase
 {
     Q_OBJECT
 
 public:
-    LabelFiltersList(QWidget *parent, TransferListWidget *transferList);
-    ~LabelFiltersList();
+    CategoryFiltersList(QWidget *parent, TransferListWidget *transferList);
 
 private slots:
     // Redefine addItem() to make sure the list stays sorted
-    void addItem(QString &label, bool hasTorrent = false);
-    void removeItem(const QString &label);
-    void removeSelectedLabel();
-    void removeUnusedLabels();
-    void torrentChangedLabel(BitTorrent::TorrentHandle *const torrent, const QString &oldLabel);
-
+    void addItem(const QString &category, bool hasTorrent = false);
+    void removeItem(const QString &category);
+    void removeSelectedCategory();
+    void removeUnusedCategories();
+    void torrentCategoryChanged(BitTorrent::TorrentHandle *const torrent, const QString &oldCategory);
+    void categoryRemoved(const QString &category);
+    void subcategoriesSupportChanged();
 
 private:
     // These 4 methods are virtual slots in the base class.
@@ -114,13 +114,14 @@ private:
     virtual void applyFilter(int row);
     virtual void handleNewTorrent(BitTorrent::TorrentHandle *const torrent);
     virtual void torrentAboutToBeDeleted(BitTorrent::TorrentHandle *const torrent);
-    QString labelFromRow(int row) const;
-    int rowFromLabel(const QString &label) const;
+    QString categoryFromRow(int row) const;
+    int rowFromCategory(const QString &category) const;
+    void refresh();
 
 private:
-    QHash<QString, int> m_labels;
+    QHash<QString, int> m_categories;
     int m_totalTorrents;
-    int m_totalLabeled;
+    int m_totalCategorized;
 };
 
 class TrackerFiltersList: public FiltersBase
@@ -154,7 +155,7 @@ private:
     virtual void torrentAboutToBeDeleted(BitTorrent::TorrentHandle *const torrent);
     QString trackerFromRow(int row) const;
     int rowFromTracker(const QString &tracker) const;
-    QString getHost(const QString &trakcer) const;
+    QString getHost(const QString &tracker) const;
     QStringList getHashes(int row);
     void downloadFavicon(const QString &url);
 
